﻿using System;
using System.IO;
using BCS.Sym3;

namespace $safeprojectname$
{
    //-- This is a description for the Add-in
    [BCS.Sym3.Sym3Addin("Demo plugin for Sym3")]
    class AddInDescription : BCS.Sym3.IAddIn
    {
        private BCS.Sym3.ISym3BaseEditor editor;
        private AddInRibbon addInRibbon;

        /// <summary>
        /// Constructor
        /// </summary>
        public AddInDescription()
        {
            addInRibbon = new AddInRibbon(this);
        }

        private String GetEditorSettingsFile()
        {
            // get XML filename:
            return System.IO.Path.Combine(UserSettingsPath, "settings.addin.demo.xml");
        }

        private String UserSettingsPath
        {
            get
            {
                String tmpStr = BCS.ToolApplication.ApplicationUserPath;

                // get company directory (create it if doesn't exist):
                tmpStr = Path.Combine(tmpStr, editor.Application.CompanyName);
                if (!Directory.Exists(tmpStr))
                {
                    Directory.CreateDirectory(tmpStr);
                }

                // get Sym3 Directory (create it if doesn't exist):
                tmpStr = Path.Combine(tmpStr, editor.Application.ProductName);
                if (!Directory.Exists(tmpStr))
                {
                    Directory.CreateDirectory(tmpStr);
                }

                return tmpStr;
            }
        }

        #region IAddin Members

        /// <summary>
        /// Get Addin name
        /// </summary>
        public string Name { get { return "Demo for Sym3"; } }

        /// <summary>
        /// Get Addin description
        /// </summary>
        public string Description { get { return "This addin provides a demo add-in for Sym3"; } }

        /// <summary>
        /// Get author name
        /// </summary>
        public string Author { get { return "Sym3 Limited"; } }

        /// <summary>
        /// Get version
        /// </summary>
        public string Version { get { return "1.0.0.0"; } }

        /// <summary>
        /// Get Addin Ribbon
        /// </summary>
        public IAddInRibbon AddInRibbon { get { return addInRibbon; } }

        /// <summary>
        /// Get access to Sym3 editor
        /// </summary>
        public BCS.Sym3.ISym3BaseEditor Editor { get { return this.editor; } }

        /// <summary>
        /// Initialize the Add-in. This function will be called by Sym3 at start-up
        /// </summary>
        /// <param name="editor"></param>
        public void Init(BCS.Sym3.ISym3BaseEditor editor)
        {
            this.editor = editor;

            if (editor == null)
                throw new Exception("Sym3 Editor is not set: " + Name);

            if (editor.Application == null)
                throw new Exception("The application has not been set for addin: " + Name);

            //---- init settings manager:
            try
            {
                Settings.Instance.Init(GetEditorSettingsFile());
            }
            catch (Exception ex)
            {

                editor.Logger.Error("Failed to load xml settings of addin " + Name + ", reason : " + ex.Message + ". Restting to default");
            }

            //---- Init Ribbon
            addInRibbon.Init();

            //---- add the panel:
            Panels.DemoToolsPanel panel = new Panels.DemoToolsPanel(editor, addInRibbon);
            editor.Application.AddDockablePanel(panel);
        }

        #endregion
    }
}
