﻿using System;
using System.Xml.Linq;
using System.Collections.Specialized;
using System.IO;
using System.Xml;

namespace $safeprojectname$
{
    class Settings
    {
        //---------------------- Settings ------------------------------
        #region Other Settings

        #endregion

        //*************************************************************************************//

        #region Singleton

        private static Settings _instance;

        Settings()
        {
            SaveAfterEachModification = true;
        }

        public static Settings Instance
        {
            get
            {
                if (_instance == null) _instance = new Settings();
                return _instance;
            }
        }

        #endregion

        #region FIELDS

        private static NameValueCollection _Settings = new NameValueCollection();
        private string xmlFilePath;

        #endregion

        #region PUBLIC

        public void Init(string path)
        {
            xmlFilePath = path;
            LoadXml();
        }

        public bool SaveAfterEachModification { get; set; }

        public void Apply()
        {
            SaveXml();
        }

        #endregion

        #region PRIVATE MEMBERS

        private string GetSetting(string name, string defaultValue)
        {
            if (SettingExists(name))
                return _Settings[name];

            return defaultValue;
        }

        private bool GetSettingBoolean(string name, bool defaultValue)
        {
            String str = GetSetting(name, defaultValue.ToString());
            bool result = defaultValue;
            try
            {
                result = Boolean.Parse(str);
            }
            catch (Exception)
            {
            }

            return result;
        }

        private Int32 GetSettingInt32(string name, Int32 defaultValue)
        {
            String str = GetSetting(name, defaultValue.ToString());
            Int32 result = defaultValue;
            try
            {
                result = Int32.Parse(str);
            }
            catch (Exception)
            {
            }

            return result;
        }

        private void SetSetting(string name, object value)
        {
            string valueString = "";
            if (value != null) valueString = value.ToString();

            if (SettingExists(name) == true)
            {
                _Settings[name] = valueString;
            }
            else
            {
                _Settings.Add(name, valueString);
            }

            if (SaveAfterEachModification == true)
                SaveXml();
        }

        private void LoadXml()
        {
            XDocument doc = null;

            if (System.IO.File.Exists(xmlFilePath) == false)
            {
                // create file
                doc = new XDocument();
                doc.Add(new XElement("Settings"));

                SaveXmlDocument(doc);
            }
            else
            {
                try
                {
                    doc = XDocument.Load(xmlFilePath);
                }
                catch (XmlException)
                {
                    //bad xml. reset to default
                    doc = new XDocument();
                    doc.Add(new XElement("Settings"));

                    SaveXmlDocument(doc);

                    throw;
                }
            }

            _Settings.Clear();
            foreach (XElement elem in doc.Root.Elements())
            {
                _Settings.Add(elem.Name.ToString(), elem.Value);
            }
        }

        private void SaveXml()
        {
            XDocument doc = new XDocument();
            doc.Add(new XElement("Settings"));

            foreach (string str in _Settings.AllKeys)
            {
                XElement newElem = new XElement(str, _Settings[str]);
                doc.Root.Add(newElem);
            }

            SaveXmlDocument(doc);
        }

        private void SaveXmlDocument(XDocument doc)
        {
            try
            {
                string dir = System.IO.Path.GetDirectoryName(xmlFilePath);
                if (System.IO.Directory.Exists(dir) == false)
                {
                    // create directory:
                    System.IO.Directory.CreateDirectory(dir);
                }

                doc.Save(xmlFilePath);

            }
            catch (Exception)
            {
            }
        }

        private bool SettingExists(string key)
        {
            return (_Settings[key] != null);
        }

        
        #endregion
    }
}
